/*
 Alarm System Monitor - Server
 Version: 2.2
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/08/2012
 Last Modified: 7/10/2014
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1
#define pass 1
#define fail 2

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

#define SHIFT_DATA PC0
#define SHIFT_CLOCK PC1
#define SHIFT_LATCH PC2
#define CHECK_IN_OK PC3
#define CHECK_IN_FAIL PC4
#define SIREN_LOCAL PB0
#define SYSTEM_ON_LED PD7

#define lowVoltagePIR 585
#define lowVoltageSiren 227
#define lowVoltageDoor 400

#define firstBootlocation 6
#define blankEeprombyte 255
volatile uint8_t firstBoot = blankEeprombyte;

#define SENSOR_CODE_ADDITION_LOCATION 0
#define RC_CODE_ADDITION_LOCATION 2
extern uint8_t sensor_code[];
extern uint16_t sensor_code_addition;
extern uint8_t rc_code[];
extern uint32_t rc_code_addition;

extern void mirf_init(void);
extern void mirf_config(void);

// Transmit USART data
void USART_Transmit(unsigned char data) {
	while ( !( UCSR0A & (1<<UDRE0)) ); // Wait for empty transmit buffer
	UDR0 = data;
}

// Shift out to 74HC595
void shift_out(uint8_t data) {
	for (uint8_t i = 0; i < 8; i++) {
		if (data & (1<<i)) {
			PORTC |= (1<<SHIFT_DATA);
		} else {
			PORTC &= ~(1<<SHIFT_DATA);
		}
		PORTC |= (1<<SHIFT_CLOCK);
		PORTC &= ~(1<<SHIFT_CLOCK);
	}
}

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	SPDR = data;
	while (!(SPSR & (1<<SPIF)));
	return SPDR;
}

// Write data using SPI
void spi_write_data(uint8_t * dataout, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		spi_transfer(dataout[i]);
	}
}

// Read data using SPI
void spi_read_data(uint8_t * datain, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		datain[i] = spi_transfer(0x00);
	}
}

// Sleep mode
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(uint8_t ii) {
	if (ii > 9) { ii = 9; }
	uint8_t bb = ii & 7;
	if (ii > 7) { bb |= (1<<5); }
	bb |= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = bb;
	WDTCSR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(int timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}

// Initialise the SPI
void spi_init(void) {
	DDRB |= (1<<PB5); // SPI CLK
	DDRB |= (1<<PB3); // SPI DO
	DDRB &= ~(1<<PB4); // SPI DI
	PORTB |= (1<<PB4); // SPI DI
	
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
}

// Setup the MCU
void setup(void) {
	_delay_ms(250); // Wait a little bit after power on
	mirf_init(); // Initilise CSN and CE as outputs before initialising SPI
	spi_init(); // Initialise SPI
	mirf_config(); // mIRF configure
	
	// Configure outputs
	DDRC |= ((1<<SHIFT_DATA) | (1<<SHIFT_CLOCK) | (1<<SHIFT_LATCH));
	DDRC |= ((1<<CHECK_IN_OK) | (1<<CHECK_IN_FAIL));
	DDRB |= (1<<SIREN_LOCAL);	
	
	// Timer0 setup for check in LEDs, every 20ms
	TCNT0 = 0; // Reset to 0
	TCCR0B = ((1<<CS02) | (1<<CS00)); // 256 prescaler
	sbi(TIMSK0, TOIE0); // Enable overflow interrupt
	
	// Timer1 setup for sensor LEDs, every 340ms
	TCNT1 = 0; // Reset to 0
	TCCR1B = ((1<<CS11) | (1<<CS10)); // 64 prescaler
	sbi(TIMSK1, TOIE1); // Enable overflow interrupt
	
	// Clear data on shift registers
	PORTC &= ~(1<<SHIFT_LATCH);
	shift_out(0);
	shift_out(0);
	PORTC |= (1<<SHIFT_LATCH);
	
	// Setup USART for Nokia SMS Sender trigger
	UBRR0L = 12; // 115,200 Baud rate
	sbi(UCSR0A, U2X0); // Double rate
	sbi(UCSR0B, TXEN0); // Transmitter enable
	
	// Check if this isn't the first boot of the MCU after programming, if it isn't then read the rolling code additiona from EEPROM
	firstBoot = eeprom_read_byte((uint8_t*) firstBootlocation);
	if (firstBoot != blankEeprombyte) {
		// Read the sensor rolling code from EEPROM
		sensor_code_addition = ((eeprom_read_byte((uint8_t*) SENSOR_CODE_ADDITION_LOCATION)) << 8) | eeprom_read_byte((uint8_t*) SENSOR_CODE_ADDITION_LOCATION+1);
		
		// Initialise the rolling code
		for (uint16_t z = 0; z < sensor_code_addition; z++) {
			sensor_code[15]++;
			if (sensor_code[15] == 0) {
				for (uint8_t a = 15; a >= 0; a--) {
					if (sensor_code[a] == 0) {
						sensor_code[(a-1)]++;
					}
					else {
						break;
					}
				}
			}
		}
		
		// Read the sensor rolling code from EEPROM
		rc_code_addition = (uint32_t) eeprom_read_byte((uint8_t*) RC_CODE_ADDITION_LOCATION) << 24 | (uint32_t) eeprom_read_byte((uint8_t*) RC_CODE_ADDITION_LOCATION+1) << 16 | (uint32_t) eeprom_read_byte((uint8_t*) RC_CODE_ADDITION_LOCATION+2) << 8 | (uint32_t) eeprom_read_byte((uint8_t*) RC_CODE_ADDITION_LOCATION+3);
		
		// Initialise the rolling code
		for (uint32_t z = 0; z < rc_code_addition; z++) {
			rc_code[15]++;
			if (rc_code[15] == 0) {
				for (uint8_t a = 15; a >= 0; a--) {
					if (rc_code[a] == 0) {
						rc_code[(a-1)]++;
					}
					else {
						break;
					}
				}
			}
		}
	}
	
	sei(); // Turn on interrupts
}
